"""Load rasters into Geoserver."""
import os
import sys
import logging
import argparse
import glob
import re

import requests

LOGGER = logging.getLogger(__name__)

logging.basicConfig(
    level=logging.DEBUG,
    format=('%(message)s'),
    stream=sys.stdout)
LOGGER = logging.getLogger(__name__)

REQUEST_TIMEOUT = 5.0


def main():
    """Entry point."""
    parser = argparse.ArgumentParser(description='Ecoshard geoserver.')
    parser.add_argument('username', help='username')
    parser.add_argument('password', help='password')
    args = parser.parse_args()

    session = requests.Session()
    session.auth = (args.username, args.password)

    path_prefix = '/mnt/disks/geoserver_data/'
    for filepath in glob.glob(
            '/mnt/disks/geoserver_data/data/*.tif'):
        sub_path = filepath[len(path_prefix):]
        LOGGER.debug(filepath)
        LOGGER.debug(sub_path)
        compress_match = re.match(
            '^(.*)_compressed', os.path.basename(sub_path))
        if compress_match:
            raster_name = compress_match.group(1)
        else:
            raster_name = compress_match = re.match(
                '^(.*)_md5', os.path.basename(sub_path)).group(1)
        payload = {
            "coverageStore": {
                "name": raster_name,
                "url": "file:%s" % sub_path
            }
        }
        url = 'http://localhost:8080/geoserver/rest/workspaces/ipbes/coveragestores'
        response = requests.post(url, json=payload)
        LOGGER.info(response.text)
        break


if __name__ == '__main__':
    main()



"""Main ecoshard module."""
import subprocess
import urllib.request
import time
import datetime
import shutil
import hashlib
import re
import os
import logging

import numpy
import scipy.stats
from osgeo import gdal
import pygeoprocessing

LOGGER = logging.getLogger(__name__)


def hash_file(
        base_path, target_token_path=None, target_dir=None, rename=False,
        hash_algorithm='md5', force=False):
    """Ecoshard file by hashing it and appending hash to filename.

    An EcoShard is the hashing of a file and the rename to the following
    format: [base name]_[hashalg]_[hash][base extension]. If the base path
    already is in this format a ValueError is raised unless `force` is True.

    Parameters:
        base_path (str): path to base file.
        target_token_path (str): if not None, this file is created and written
            with the timestamp at which the ecoshard was completed. This is
            useful for TaskGraph to note a file being created without a priori
            knowing the filename.
        target_dir (str): if present, the ecoshard is created in this
            directory. This value must be None if `rename` is True.
        rename (bool): if True, `base_path` is renamed to the ecoshard rather
            than a new file being created.
        hash_algorithm (str): a hash function id that exists in
            hashlib.algorithms_available.
        force (bool): if True and the base_path already is in ecoshard format
            the operation proceeds including the possibility that the
            base_path ecoshard file name is renamed to a new hash.

    Returns:
        None.

    """
    if target_dir and rename:
        raise ValueError(
            "`target_dir` is defined, but rename is True, either set "
            "`target_dir` to None, or rename to False.")

    if target_dir and not os.path.isdir(target_dir):
        LOGGER.warning('target directory %s does not exist, creating it now')
        try:
            os.makedirs(target_dir)
        except OSError:
            # this would never happen unless there was some concurrency that
            # created the target dir after the test, this guards against it.
            LOGGER.exception('failed to make %s', target_dir)

    base_filename = os.path.basename(base_path)
    prefix, extension = os.path.splitext(base_filename)
    match_result = re.match(
        '(.+)_([^_]+)_([0-9a-f]+)%s' % extension, base_filename)
    if match_result:
        if not force:
            raise ValueError(
                '%s seems to already be an ecoshard with algorithm %s and '
                'hash %s. Set `force=True` to overwrite.' % (
                    base_path, match_result.group(1), match_result.group(2)))
        else:
            LOGGER.warning(
                '%s is already in ecoshard format, but overriding because '
                '`force` is True.', base_path)
            prefix = match_result.group(1)

    LOGGER.debug('calculating hash for %s', base_path)
    hash_val = calculate_hash(base_path, hash_algorithm)

    if target_dir is None:
        target_dir = os.path.dirname(base_path)
    ecoshard_path = os.path.join(target_dir, '%s_%s_%s%s' % (
        prefix, hash_algorithm, hash_val, extension))
    if rename:
        LOGGER.info('renaming %s to %s', base_path, ecoshard_path)
        os.rename(base_path, ecoshard_path)
    else:
        LOGGER.info('copying %s to %s', base_path, ecoshard_path)
        shutil.copyfile(base_path, ecoshard_path)

    if target_token_path:
        with open(target_token_path, 'w') as target_token_file:
            target_token_file.write(str(datetime.datetime.now()))


def build_overviews(
        base_raster_path, target_token_path=None,
        interpolation_method='near'):
    """Build embedded overviews on raster.

    Parameters:
        base_raster_path (str): base raster file, must be a GDAL writable
            raster type.
        target_token_path (str): if not None, this file is created and written
            with a timestamp when overviews are successfully completed. This
            file is useful for a library like `taskgraph` that needs to see
            a file to know if an operation is successful.
        interpolation_method (str): one of 'average', 'average_magphase',
            'bilinear', 'cubic', 'cubicspline', 'gauss', 'lanczos', 'mode',
            'near', or 'none'.

    Returns:
        None.

    """
    raster = gdal.OpenEx(base_raster_path, gdal.OF_RASTER | gdal.GA_Update)
    if not raster:
        raise ValueError(
            'could not open %s as a GDAL raster' % base_raster_path)
    band = raster.GetRasterBand(1)
    overview_count = band.GetOverviewCount()
    if overview_count != 0:
        LOGGER.warn(
            '%d overviews already exist for %s still creating anyway',
            overview_count, base_raster_path)
    min_dimension = min(raster.RasterXSize, raster.RasterYSize)
    overview_levels = []
    current_level = 2
    while True:
        if min_dimension // current_level == 0:
            break
        overview_levels.append(current_level)
        current_level *= 2

    LOGGER.info(
        'building overviews for %s at the following levels %s' % (
            base_raster_path, overview_levels))
    raster.BuildOverviews(
        'average', overview_levels, callback=_make_logger_callback(
            'build overview for ' + os.path.basename(base_raster_path) +
            '%.2f/1.0 complete'))

    if target_token_path:
        with open(target_token_path, 'w') as token_file:
            token_file.write(str(datetime.datetime.now()))


def validate(base_ecoshard_path):
    """Validate ecoshard path, through its filename.

    If `base_ecoshard_path` matches an EcoShard pattern, and the hash matches
    the actual hash, return True. Otherwise raise a ValueError.

    Parameters:
        base_ecoshard_path (str): path to an ecosharded file.

    Returns:
        True if `base_ecoshard_path` matches .*_[hashalg]_[hash][extension]
        and hashalg(base_ecoshard_path) = hash. Otherwise raise a ValueError.

    """
    base_filename = os.path.basename(base_ecoshard_path)
    prefix, extension = os.path.splitext(base_filename)
    match_result = re.match(
        '.+_([^_]+)_([0-9a-f]+)%s' % extension, base_filename)
    if not match_result:
        raise ValueError("%s does not match an ecoshard" % base_filename)
    hash_algorithm, hash_value = match_result.groups()
    calculated_hash = calculate_hash(
        base_ecoshard_path, hash_algorithm)
    if calculated_hash != match_result.group(2):
        raise ValueError(
            'hash does not match, calculated %s and expected %s '
            'on %s' % (calculated_hash, hash_value, base_filename))
    # if we got here the hash matched the calculated hash
    return True


def calculate_hash(file_path, hash_algorithm, buf_size=2**20):
    """Return a hex digest of `file_path`.

    Parameters:
        file_path (string): path to file to hash.
        hash_algorithm (string): a hash function id that exists in
            hashlib.algorithms_available.
        buf_size (int): number of bytes to read from `file_path` at a time
            for digesting.

    Returns:
        a hex digest with hash algorithm `hash_algorithm` of the binary
        contents of `file_path`.

    """
    hash_func = hashlib.new(hash_algorithm)
    with open(file_path, 'rb') as f:
        binary_data = f.read(buf_size)
        while binary_data:
            hash_func.update(binary_data)
            binary_data = f.read(buf_size)
    # We return the hash and CRC32 checksum in hexadecimal format
    return hash_func.hexdigest()


def _make_logger_callback(message):
    """Build a timed logger callback that prints ``message`` replaced.

    Parameters:
        message (string): a string that expects 2 placement %% variables,
            first for % complete from ``df_complete``, second from
            ``p_progress_arg[0]``.

    Returns:
        Function with signature:
            logger_callback(df_complete, psz_message, p_progress_arg)

    """
    def logger_callback(df_complete, _, p_progress_arg):
        """Argument names come from the GDAL API for callbacks."""
        try:
            current_time = time.time()
            if ((current_time - logger_callback.last_time) > 5.0 or
                    (df_complete == 1.0 and
                     logger_callback.total_time >= 5.0)):
                # In some multiprocess applications I was encountering a
                # ``p_progress_arg`` of None. This is unexpected and I suspect
                # was an issue for some kind of GDAL race condition. So I'm
                # guarding against it here and reporting an appropriate log
                # if it occurs.
                if p_progress_arg:
                    LOGGER.info(message, df_complete * 100, p_progress_arg[0])
                else:
                    LOGGER.info(
                        'p_progress_arg is None df_complete: %s, message: %s',
                        df_complete, message)
                logger_callback.last_time = current_time
                logger_callback.total_time += current_time
        except AttributeError:
            logger_callback.last_time = time.time()
            logger_callback.total_time = 0.0

    return logger_callback


def compress_raster(
        base_raster_path, target_compressed_path, compression_algorithm='LZW',
        compression_predictor=None):
    """Compress base raster to target.

    Parameters:
        base_raster_path (str): the original GIS raster file, presumably
            uncompressed.
        target_compressed_path (str): the desired output raster path with the
            defined compression algorithm applied to it.
        compression_algorithm (str): a valid GDAL compression algorithm eg
            'LZW', 'DEFLATE', and others defined in GDAL.
        compression_predictor (int): if defined uses the predictor in whatever
            compression algorithm is used. In most cases this only applies to
            LZW or DEFLATE.

    Returns:
        None.

    """
    gtiff_driver = gdal.GetDriverByName('GTiff')
    base_raster = gdal.OpenEx(base_raster_path, gdal.OF_RASTER)
    LOGGER.info('compress %s to %s' % (
        base_raster_path, target_compressed_path))
    compressed_raster = gtiff_driver.CreateCopy(
        target_compressed_path, base_raster, options=(
            'TILED=YES', 'BIGTIFF=YES', 'COMPRESS=%s' % compression_algorithm,
            'BLOCKXSIZE=256', 'BLOCKYSIZE=256'))
    del compressed_raster


def download_url(url, target_path, skip_if_target_exists=False):
    """Download `url` to `target_path`.

    Parameters:
        url (str): url path to a file.
        target_path (str): desired output target path.
        skip_if_target_exists (bool): if True will not download a file if the
            path already exists on disk.

    Returns:
        None.

    """
    if skip_if_target_exists and os.path.exists(target_path):
        return
    with open(target_path, 'wb') as target_file:
        last_download_size = 0
        last_log_time = 0
        with urllib.request.urlopen(url) as url_stream:
            meta = url_stream.info()
            file_size = int(meta["Content-Length"])
            LOGGER.info(
                "Downloading: %s Bytes: %s" % (target_path, file_size))

            downloaded_so_far = 0
            block_size = 2**20
            while True:
                data_buffer = url_stream.read(block_size)
                if not data_buffer:
                    break
                last_download_size = downloaded_so_far
                downloaded_so_far += len(data_buffer)
                target_file.write(data_buffer)
                time_since_last_log = time.time() - last_log_time
                if time_since_last_log > 5.0:
                    download_rate = (
                        (downloaded_so_far - last_download_size)/2**20) / (
                        time_since_last_log)
                    status = r"%10dMB  [%3.2f%% @ %5.2fMB/s]" % (
                        downloaded_so_far/2**20, downloaded_so_far * 100. /
                        file_size, download_rate)
                    LOGGER.info(status)
                    last_log_time = time.time()


def copy_to_bucket(base_path, target_gs_path, target_token_path=None):
    """Copy base to a Google Bucket path.

    This requires that "gsutil" is installed on the host machine and the
    client has write access to whatever gs path is written.

    Parameters:
        base_path (str): path to base file.
        target_gs_path (str): a well formated google bucket string of the
            format "gs://[bucket][path][file]"
        target_token_path (str): file that is written if this operation
            completes successfully, contents are the timestamp of the
            creation time.

    Returns:
        None.

    """
    subprocess.run(
        'gsutil cp -n %s %s' % (base_path, target_gs_path), shell=True,
        check=True)
    if target_token_path:
        with open(target_token_path, 'w') as token_file:
            token_file.write(str(datetime.datetime.now()))


def convolve_layer(
        base_raster_path, integer_factor, method, target_raster_path):
    """Convolve a raster to a lower size.

    Parameters:
        base_raster_path (str): base raster.
        integer_factor (int): integer number of pixels to aggregate by.
            i.e. 2 -- makes 2x2 into a 1x1, 3-- 3x3 to a 1x1.
        method (str): one of 'max', 'min', 'sum', 'average', 'mode'.
        target_rater_path (str): based off of `base_raster_path` with size
            reduced by `integer_factor`.

    Return:
        None.

    """
    base_raster_info = pygeoprocessing.get_raster_info(base_raster_path)
    n_cols, n_rows = numpy.ceil(base_raster_info['raster_size']).astype(
        numpy.int)
    n_cols_reduced = int(numpy.ceil(n_cols / integer_factor))
    n_rows_reduced = int(numpy.ceil(n_rows / integer_factor))
    nodata = base_raster_info['nodata'][0]
    pygeoprocessing.new_raster_from_base(
        base_raster_path, target_raster_path, base_raster_info['datatype'],
        [nodata], n_rows=n_rows_reduced, n_cols=n_cols_reduced)

    base_raster = gdal.OpenEx(base_raster_path, gdal.OF_RASTER)
    base_band = base_raster.GetRasterBand(1)
    base_geotransform = base_raster.GetGeoTransform()
    target_raster = gdal.OpenEx(
        target_raster_path, gdal.OF_RASTER | gdal.GA_Update)
    target_geotransform = [
        base_geotransform[0],
        base_geotransform[1]*integer_factor,
        base_geotransform[2]*integer_factor,
        base_geotransform[3],
        base_geotransform[4]*integer_factor,
        base_geotransform[5]*integer_factor]
    target_raster.SetGeoTransform(target_geotransform)

    target_band = target_raster.GetRasterBand(1)

    block = base_band.GetBlockSize()
    cols_per_block = min(
        n_cols, max(1, block[0] // integer_factor) * integer_factor * 10)
    rows_per_block = min(
        n_rows, max(1, block[1] // integer_factor) * integer_factor * 10)
    n_col_blocks = int(numpy.ceil(n_cols / float(cols_per_block)))
    n_row_blocks = int(numpy.ceil(n_rows / float(rows_per_block)))
    for row_block_index in range(n_row_blocks):
        row_offset = row_block_index * rows_per_block
        row_block_width = n_rows - row_offset
        LOGGER.info('step %d of %d', row_block_index+1, n_row_blocks)
        if row_block_width > rows_per_block:
            row_block_width = rows_per_block
        for col_block_index in range(n_col_blocks):
            col_offset = col_block_index * cols_per_block
            col_block_width = n_cols - col_offset
            if col_block_width > cols_per_block:
                col_block_width = cols_per_block

            offset_dict = {
                'xoff': int(col_offset),
                'yoff': int(row_offset),
                'win_xsize': int(col_block_width),
                'win_ysize': int(row_block_width),
            }

            target_offset_x = offset_dict['xoff'] // integer_factor
            target_offset_y = offset_dict['yoff'] // integer_factor

            block_data = base_band.ReadAsArray(**offset_dict)

            rw = int(numpy.ceil(col_block_width / integer_factor) * integer_factor)
            rh = int(numpy.ceil(row_block_width / integer_factor) * integer_factor)
            w_pad = rw - col_block_width
            h_pad = rh - row_block_width
            j = rw // integer_factor
            k = rh // integer_factor
            if method == 'max':
                block_data_pad = numpy.pad(
                    block_data, ((0, h_pad), (0, w_pad)), mode='edge')
                reduced_block_data = block_data_pad.reshape(
                    k, integer_factor, j, integer_factor).max(axis=(-1, -3))
            elif method == 'min':
                block_data_pad = numpy.pad(
                    block_data, ((0, h_pad), (0, w_pad)), mode='edge')
                reduced_block_data = block_data_pad.reshape(
                    k, integer_factor, j, integer_factor).min(axis=(-1, -3))
            elif method == 'mode':
                block_data_pad = numpy.pad(
                    block_data, ((0, h_pad), (0, w_pad)), mode='edge')
                reduced_block_data = scipy.stats.mode(
                    block_data_pad.reshape(
                        k, integer_factor, j, integer_factor).swapaxes(
                            1, 2).reshape(k, j, integer_factor**2),
                    axis=2).mode.reshape(k, j)
            elif method == 'average':
                block_data_pad = numpy.pad(
                    block_data, ((0, h_pad), (0, w_pad)), mode='edge')
                block_data_pad_copy = block_data_pad.copy()
                # set any nodata to 0 so we don't average it strangely
                block_data_pad[numpy.isclose(block_data_pad, nodata)] = 0.0
                # straight average
                reduced_block_data = block_data_pad.reshape(
                    k, integer_factor, j, integer_factor).mean(
                    axis=(-1, -3))
                # this one is used to restore any nodata areas because they'll
                # still be nodata when it's done
                min_block_data = block_data_pad_copy.reshape(
                    k, integer_factor, j, integer_factor).min(
                    axis=(-1, -3))
                reduced_block_data[
                    numpy.isclose(min_block_data, nodata)] = nodata
            elif method == 'sum':
                block_data_pad = numpy.pad(
                    block_data, ((0, h_pad), (0, w_pad)), mode='edge')
                block_data_pad_copy = block_data_pad.copy()
                # set any nodata to 0 so we don't sum it strangely
                block_data_pad[numpy.isclose(block_data_pad, nodata)] = 0.0
                # straight sum
                reduced_block_data = block_data_pad.reshape(
                    k, integer_factor, j, integer_factor).sum(
                    axis=(-1, -3))
                # this one is used to restore any nodata areas because they'll
                # still be nodata when it's done
                min_block_data = block_data_pad_copy.reshape(
                    k, integer_factor, j, integer_factor).min(
                    axis=(-1, -3))
                reduced_block_data[
                    numpy.isclose(min_block_data, nodata)] = nodata
            else:
                raise ValueError("unknown method: %s" % method)

            target_band.WriteArray(
                reduced_block_data, xoff=target_offset_x, yoff=target_offset_y)
            continue



"""Ecoshard test suite."""
import os
import tempfile
import shutil
import unittest

import ecoshard
import numpy
from osgeo import gdal
from osgeo import osr


def _build_test_raster(raster_path):
    srs = osr.SpatialReference()
    srs.ImportFromEPSG(4326)

    gtiff_driver = gdal.GetDriverByName('GTiff')
    n = 100
    new_raster = gtiff_driver.Create(
        raster_path, n, n, 1, gdal.GDT_Int32, options=[
            'TILED=YES', 'BIGTIFF=YES', 'COMPRESS=NONE',
            'BLOCKXSIZE=16', 'BLOCKYSIZE=16'])
    new_raster.SetProjection(srs.ExportToWkt())
    new_raster.SetGeoTransform([100.0, 1.0, 0.0, 100.0, 0.0, -1.0])
    new_band = new_raster.GetRasterBand(1)
    new_band.SetNoDataValue(-1)
    array = numpy.array(range(n*n), dtype=numpy.int32).reshape((n, n))
    new_band.WriteArray(array)
    new_raster.FlushCache()
    new_band = None
    new_raster = None


class EcoShardTests(unittest.TestCase):
    """Tests for the PyGeoprocesing 1.0 refactor."""

    def setUp(self):
        """Create a temporary workspace that's deleted later."""
        self.workspace_dir = tempfile.mkdtemp()

    def tearDown(self):
        """Clean up remaining files."""
        shutil.rmtree(self.workspace_dir)

    def test_hash_file(self):
        """Test ecoshard.hash_file."""
        working_dir = self.workspace_dir
        base_path = os.path.join(working_dir, 'test_file.txt')
        target_token_path = '%s.COMPLETE' % base_path

        with open(base_path, 'w') as base_file:
            base_file.write('test')

        # make a different target dir so it gets copied
        target_dir = os.path.join(working_dir, 'output')
        ecoshard.hash_file(
            base_path, target_token_path=target_token_path,
            target_dir=target_dir, rename=False,
            hash_algorithm='md5', force=False)

        expected_file_path = os.path.join(
            target_dir, 'test_file_md5_098f6bcd4621d373cade4e832627b4f6.txt')
        self.assertTrue(os.path.exists(expected_file_path))
        self.assertTrue(os.path.exists(target_token_path))

    def test_hash_file_rename(self):
        """Test ecoshard.hash_file with a rename."""
        working_dir = self.workspace_dir
        base_path = os.path.join(working_dir, 'test_file.txt')
        target_token_path = '%s.COMPLETE' % base_path

        with open(base_path, 'w') as base_file:
            base_file.write('test')

        ecoshard.hash_file(
            base_path, target_token_path=target_token_path,
            target_dir=None, rename=True,
            hash_algorithm='md5', force=False)

        expected_file_path = os.path.join(
            working_dir, 'test_file_md5_098f6bcd4621d373cade4e832627b4f6.txt')
        self.assertTrue(os.path.exists(expected_file_path))
        # indicates the file has been renamed
        self.assertFalse(os.path.exists(base_path))

    def test_force(self):
        """Test ecoshard.hash_file with a force rename."""
        working_dir = self.workspace_dir
        base_path = os.path.join(working_dir, 'test_file_sha_fffffffffff.txt')
        target_token_path = '%s.COMPLETE' % base_path

        with open(base_path, 'w') as base_file:
            base_file.write('test')

        ecoshard.hash_file(
            base_path, target_token_path=target_token_path,
            target_dir=None, rename=True,
            hash_algorithm='md5', force=True)

        expected_file_path = os.path.join(
            working_dir, 'test_file_md5_098f6bcd4621d373cade4e832627b4f6.txt')
        self.assertTrue(os.path.exists(expected_file_path))
        # indicates the file has been renamed
        self.assertFalse(os.path.exists(base_path))

    def test_exceptions_in_hash(self):
        """Test ecoshard.hash_file raises exceptions in bad cases."""
        working_dir = self.workspace_dir
        base_path = os.path.join(working_dir, 'test_file.txt')
        target_token_path = '%s.COMPLETE' % base_path

        with open(base_path, 'w') as base_file:
            base_file.write('test')

        # test that target dir defined and rename True raises an exception
        with self.assertRaises(ValueError) as cm:
            ecoshard.hash_file(
                base_path, target_token_path=target_token_path,
                target_dir='output', rename=True,
                hash_algorithm='md5', force=False)
        self.assertTrue('but rename is True' in str(cm.exception))

        # test that a base path already in ecoshard format raises an exception
        base_path = os.path.join(working_dir, 'test_file_sha_fffffffffff.txt')
        with self.assertRaises(ValueError) as cm:
            ecoshard.hash_file(
                base_path, target_token_path=target_token_path,
                target_dir=None, rename=True,
                hash_algorithm='md5', force=False)
        self.assertTrue('already be an ecoshard' in str(cm.exception))

    def test_validate_hash(self):
        """Test ecoshard.validate_hash."""
        working_dir = self.workspace_dir
        # we know the hash a priori, just make the file
        base_path = os.path.join(
            working_dir, 'test_file_md5_098f6bcd4621d373cade4e832627b4f6.txt')
        with open(base_path, 'w') as base_file:
            base_file.write('test')
        self.assertTrue(ecoshard.validate(base_path))

        # test that files that are not in ecoshard format raise an exception
        with self.assertRaises(ValueError) as cm:
            not_an_ecoshard_path = 'test.txt'
            ecoshard.validate(not_an_ecoshard_path)
        self.assertTrue('does not match an ecoshard' in str(cm.exception))

        with self.assertRaises(ValueError) as cm:
            new_file_path = os.path.join(
                working_dir,
                'test_file_md5_098f6bcd4621d373cade4e832627b4f5.txt')
            shutil.copyfile(base_path, new_file_path)
            ecoshard.validate(new_file_path)
        self.assertTrue('hash does not match' in str(cm.exception))

    def test_build_overviews(self):
        """Test ecoshard.build_overviews."""
        raster_path = os.path.join(self.workspace_dir, 'test_raster.tif')
        _build_test_raster(raster_path)

        target_token_path = '%s.COMPLETE' % raster_path
        ecoshard.build_overviews(
            raster_path, target_token_path=target_token_path,
            interpolation_method='near')
        raster = gdal.OpenEx(raster_path, gdal.OF_RASTER)
        band = raster.GetRasterBand(1)
        overview_count = band.GetOverviewCount()
        band = None
        raster = None
        self.assertEqual(overview_count, 6)

    def test_compress_raster(self):
        """Test ecoshard.compress_raster."""
        raster_path = os.path.join(self.workspace_dir, 'test_raster.tif')
        _build_test_raster(raster_path)
        compressed_raster_path = os.path.join(
            self.workspace_dir, 'test_raster_compressed.tif')

        ecoshard.compress_raster(
            raster_path, compressed_raster_path, compression_algorithm='LZW',
            compression_predictor=2)

        # if its compressed, it should be smaller!
        self.assertTrue(
            os.path.getsize(compressed_raster_path) <
            os.path.getsize(raster_path))



"""EcoShard: gis file hashing and database routines.

__init__ module imports all the ecoshard functions into this namespace.
"""
from __future__ import absolute_import

import types
import sys

from . import ecoshard

__all__ = tuple()
for attrname in dir(ecoshard):
    attribute = getattr(ecoshard, attrname)
    if isinstance(attribute, types.FunctionType):
        __all__ += (attrname,)
        setattr(sys.modules['ecoshard'], attrname, attribute)



"""Load styles into Geoserver."""
import os
import sys
import logging
import argparse
import glob

import requests

LOGGER = logging.getLogger(__name__)

logging.basicConfig(
    level=logging.DEBUG,
    format=('%(message)s'),
    stream=sys.stdout)
LOGGER = logging.getLogger(__name__)

REQUEST_TIMEOUT = 5.0


def main():
    """Entry point."""
    parser = argparse.ArgumentParser(description='Ecoshard geoserver.')
    parser.add_argument('username', help='username')
    parser.add_argument('password', help='password')
    args = parser.parse_args()

    session = requests.Session()
    session.auth = (args.username, args.password)

    parent_path = '/mnt/disks/geoserver_data/'
    for filepath in glob.glob(
            '/mnt/disks/geoserver_data/data/cv_sld_files/*.sld'):
        style_path = filepath[len(parent_path):]
        LOGGER.debug(style_path)
        style_name = os.path.splitext(os.path.basename(filepath))[0]
        payload = {
          "style": {
            "name": style_name,
            "filename": filepath
          }
        }
        url = 'http://localhost:8080/geoserver/rest/workspaces/ipbes_cv_styles/styles'
        response = session.post(url, json=payload)
        LOGGER.info(response.text)

        with open(filepath, 'r') as sld_file:
            sld_payload = sld_file.read()
        LOGGER.debug(payload)
        url = 'http://localhost:8080/geoserver/rest/workspaces/ipbes_cv_styles/styles/%s' % style_name

        response = session.put(
            url,
            data=sld_payload,
            headers={
             'content-type': 'application/vnd.ogc.sld+xml',
            })
        LOGGER.info(response.text)
        """
        url = 'http://localhost:8080/geoserver/rest/layers/cv_coastal_points_output_md5_69641307c3c7b4c7d23faa8637e30f83:CV_outputs/styles'
        style_body = {
            "name": style_name,
            "filename": filepath,
        }
        # put the actual style value
        response = requests.post(url, json=style_body)
        LOGGER.info(response.text)
        """

    #LOGGER.info(response)
    #LOGGER.info(response.json())


if __name__ == '__main__':
    main()



"""Entry point for ecoshard."""
import os
import argparse
import sys
import logging
import glob

import ecoshard

LOGGER = logging.getLogger(__name__)

logging.basicConfig(
    level=logging.DEBUG,
    format=('%(message)s'),
    stream=sys.stdout)
logging.getLogger('ecoshard').setLevel(logging.INFO)
LOGGER = logging.getLogger(__name__)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Ecoshard files.')
    parser.add_argument(
        'filepath', nargs='+', help='Files/patterns to ecoshard.')
    parser.add_argument(
        '--hashalg', nargs=1, default='md5',
        help='hash algorithm in hashlib.algorithms_available')

    parser.add_argument(
        '--compress', action='store_true', help='compress the raster files.')
    parser.add_argument(
        '--buildoverviews', action='store_true',
        help='build overviews on the raster files.')
    parser.add_argument(
        '--rename', action='store_true', help=(
            'if not compressing and hashing only, rename files rather than '
            'copy new ones.'))
    parser.add_argument(
        '--interpolation_method', help=(
            'used when building overviews, can be one of '
            '"average|near|mode|min|max|..."'), default='near')
    parser.add_argument(
        '--validate', action='store_true', help=(
            'validate the ecoshard rather than hash it'))
    parser.add_argument(
        '--hash_file', action='store_true', help=(
            'hash the file and and rename/copy depending on if --rename is '
            'set'))
    parser.add_argument(
        '--force', action='store_true', help=(
            'force an ecoshard to rehash even if it is already an ecoshard.'))
    parser.add_argument(
        '--reduce_factor', help=(
            "reduce size by [factor] to with [method] to [target]. "
            "[method] must be one of 'max', 'min', 'sum', 'average', 'mode'"),
        nargs=3)

    args = parser.parse_args()
    for glob_pattern in args.filepath:
        for file_path in glob.glob(glob_pattern):
            working_file_path = file_path
            LOGGER.info('processing %s', file_path)

            if args.reduce_factor:
                method = args.reduce_factor[1]
                valid_methods = ["max", "min", "sum", "average", "mode"]
                if method not in valid_methods:
                    LOGGER.error(
                        '--reduce_method must be one of %s' % valid_methods)
                    sys.exit(-1)
                ecoshard.convolve_layer(
                    file_path, int(args.reduce_factor[0]),
                    args.reduce_factor[1],
                    args.reduce_factor[2])
                continue

            if args.compress:
                prefix, suffix = os.path.splitext(file_path)
                compressed_filename = '%s_compressed%s' % (prefix, suffix)
                ecoshard.compress_raster(
                    file_path, compressed_filename,
                    compression_algorithm='DEFLATE')
                working_file_path = compressed_filename

            if args.buildoverviews:
                overview_token_path = '%s.OVERVIEWCOMPLETE' % (
                    working_file_path)
                ecoshard.build_overviews(
                    working_file_path, target_token_path=overview_token_path,
                    interpolation_method=args.interpolation_method)

            if args.validate:
                try:
                    is_valid = ecoshard.validate(working_file_path)
                    if is_valid:
                        LOGGER.info('VALID ECOSHARD: %s', working_file_path)
                    else:
                        LOGGER.error(
                            'got a False, but no ValueError on validate? '
                            'that is not impobipible?')
                except ValueError:
                    LOGGER.error('INVALID ECOSHARD: %s', working_file_path)
            elif args.hash_file:
                hash_token_path = '%s.ECOSHARDCOMPLETE' % (
                    working_file_path)
                ecoshard.hash_file(
                    working_file_path, target_token_path=hash_token_path,
                    rename=args.rename, hash_algorithm=args.hashalg,
                    force=args.force)



"""Download rasters and style files for IPBES Paper."""
import os
import subprocess

RASTER_URL_LIST = [
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_10s_cur_compressed_md5_750e58205efb24f29fb88ec282eb0143.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_10s_ssp1_md5_9b2be93d642f895d7b847765a5072737.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_10s_ssp3_md5_7b018804dd2c60367fec1a2b34a6f9e3.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_10s_ssp5_md5_0efb0f2cae142033a9f016f6b33b8a3f.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_change_10s_ssp1_compressed_md5_1ab5bd72043041dc3c46874d870141c4.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_change_10s_ssp3_compressed_md5_63e2f488dd718e5e0e4801dfb6dc0d00.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_NC_change_10s_ssp5_compressed_md5_1b0a700d4d150a5e6bd76d87f14ab538.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_10s_cur_compressed_md5_031d4bb444325835315a2cc825be3fd4.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_10s_ssp1_md5_136b1b2609481749ed963fd6bc61a240.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_10s_ssp3_md5_93c3df4e358dcde8054426ffcad4b4fe.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_10s_ssp5_md5_7a23f76129d0f3b6aa1987d534a0960e.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_change_10s_ssp1_compressed_md5_fbeb8122b920cf13c325284baeba640c.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_change_10s_ssp3_compressed_md5_6a2352b77f23421c1a48711a3e1d703a.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_deficit_change_10s_ssp5_compressed_md5_1244e34777bb5c0980e263390b10d6d0.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_30s_cur_compressed_md5_a728d722935371a17452276ba1034296.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_30s_ssp1_md5_83f3043b257de97168c51adcd9f0006c.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_30s_ssp3_md5_c64398ed9a8329b2979e209b631afddb.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_30s_ssp5_md5_a35c2fc540b3bd5f0e55201020d52224.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_change_30s_ssp1_compressed_md5_fdadefe1e84c5dcc83e76dd7cb4f1564.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_change_30s_ssp3_compressed_md5_2d886ae99fc3241c05398a8948bf0f3a.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_pop_change_30s_ssp5_compressed_md5_72a80daaaecf1480069dfb031d4eea12.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_10s_cur_compressed_md5_9e0ae4df4e399350087c1c2d00d0a1bb.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_10s_ssp1_md5_043459977567bfc837ad4208c4f90f16.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_10s_ssp3_md5_ad4decd02bec2205bf9073f075c5bda9.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_10s_ssp5_md5_3f63cebb84fa2acce1019e66e13dfb49.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_change_10s_ssp1_compressed_md5_6a5228b9630642437a12a436be4ebe41.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_change_10s_ssp3_compressed_md5_06e7e47952d6b1ec5bc784737b7989ac.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/nutrient_potential_change_10s_ssp5_compressed_md5_1d6eb591625a6563bb64c06cf0f2a5a9.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_10s_cur_compressed_md5_5bea8fbf1d97ef6c2b539c97887e05a0.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_change_10s_ssp1_compressed_md5_b731503ebc5f7e484c8935dcce6ba79c.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_change_10s_ssp3_compressed_md5_613532434a7e247e9c09a542bf853e5e.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_change_10s_ssp5_compressed_md5_099d2aaec6051be39b9cffb91fcb1260.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_10s_cur_compressed_md5_6db2748b01d6541663f0698a8f34a607.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_change_10s_ssp1_compressed_md5_7b743214d92a37712f1fa678427f95ad.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_change_10s_ssp3_compressed_md5_5a0488cac7fd542066c4b36596c67f36.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_change_10s_ssp5_compressed_md5_19f1e1f027e468ea2217903c605cd0b7.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_10s_cur_compressed_md5_20e27265b1627b6fbdb10b5fa504e745.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_abschange_10s_ssp1_md5_da5f61b9001adb2c5ba1a795718a9f90.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_abschange_10s_ssp3_md5_6309f3eedab1bde1279ce3e1560a935d.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_abschange_10s_ssp5_md5_99bf4ab9c9e93d7b7da96428062eadbc.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_10s_ssp1_md5_4d307c30e932af67d98e40f7c5d74982.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_10s_ssp3_md5_9b5af8879fd2cfdceb7485009397ebcd.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_NC_10s_ssp5_md5_561bbf17dae9d13e28ce78cb8feb32c6.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_10s_ssp1_md5_67458bc3a7eae87a02a072a8f16ff138.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_10s_ssp3_md5_22f75fb4de30195f93c49b3a3375d7f7.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_deficit_10s_ssp5_md5_37892e314197054e7b8eb87a16b4427d.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_30s_cur_md5_566d70b81ef5a1746ab5c66cbbe1d658.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_30s_ssp1_md5_6396d2fd1400feb58f645d853b52ac85.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_30s_ssp3_md5_b49a597d072d305834bfd1b6992a7b76.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_30s_ssp5_md5_858ebf9569faba854e880443baa40c68.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_change_30s_ssp1_md5_353845ac0bdc57ce67af15ae2bd9c593.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_change_30s_ssp3_md5_0e58c680d510edcb20d2118eff3a47bd.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_pop_change_30s_ssp5_md5_670ab322ab9deb6820b5e16749377204.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_10s_ssp1_md5_57c03ee9393878d56eb7d896ab20b47a.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_10s_ssp3_md5_f785e47f2a70f98b5d9deb251c42d9f5.tif',
    'https://storage.googleapis.com/ipbes-natcap-ecoshard-data-for-publication/pollination_potential_10s_ssp5_md5_4b06474ca808725f49fbaca3fee4fe85.tif',
]

STYLE_URL_LIST = [
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=13AD6Ecb_dP3cnOWqEf4qBS8EAsalSQrj',
    'https://drive.google.com/uc?export=download&id=13AD6Ecb_dP3cnOWqEf4qBS8EAsalSQrj',
    'https://drive.google.com/uc?export=download&id=13AD6Ecb_dP3cnOWqEf4qBS8EAsalSQrj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1IUaVxt-59YieFb83d0q1DkOFBezmB_1h',
    'https://drive.google.com/uc?export=download&id=1IUaVxt-59YieFb83d0q1DkOFBezmB_1h',
    'https://drive.google.com/uc?export=download&id=1IUaVxt-59YieFb83d0q1DkOFBezmB_1h',
    'https://drive.google.com/uc?export=download&id=1IUaVxt-59YieFb83d0q1DkOFBezmB_1h',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=13AD6Ecb_dP3cnOWqEf4qBS8EAsalSQrj',
    'https://drive.google.com/uc?export=download&id=13AD6Ecb_dP3cnOWqEf4qBS8EAsalSQrj',
    'https://drive.google.com/uc?export=download&id=13AD6Ecb_dP3cnOWqEf4qBS8EAsalSQrj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1zwnCMsWMPJBAC4EuuJiSNvdl4AM8OA5W',
    'https://drive.google.com/uc?export=download&id=1zwnCMsWMPJBAC4EuuJiSNvdl4AM8OA5W',
    'https://drive.google.com/uc?export=download&id=1zwnCMsWMPJBAC4EuuJiSNvdl4AM8OA5W',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=1gLvZe_u3R31fuTc91H3XwpF0vvAb1m-Q',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1ZfTqGX8LGpX-pNXOOVjTHmen2BE7MMnm',
    'https://drive.google.com/uc?export=download&id=1ZfTqGX8LGpX-pNXOOVjTHmen2BE7MMnm',
    'https://drive.google.com/uc?export=download&id=1ZfTqGX8LGpX-pNXOOVjTHmen2BE7MMnm',
    'https://drive.google.com/uc?export=download&id=1ZfTqGX8LGpX-pNXOOVjTHmen2BE7MMnm',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1fm-0eGCLhF6YAQINjPWMIFmSGz1ehpL7',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj',
    'https://drive.google.com/uc?export=download&id=1-rs2XT0n4KkiiQO2eNxA-eoukGT5oBqj'
]

if __name__ == '__main__':
    for raster_url, style_url in zip(RASTER_URL_LIST, STYLE_URL_LIST):
        target_raster_path = os.path.basename(raster_url)
        subprocess.run('curl -L -o %s %s' % (target_raster_path, raster_url), shell=True)
        target_style_path = '%s.qml ' % (
            os.path.splitext(target_raster_path)[0])
        subprocess.run('curl -L -o %s %s' % (target_style_path, style_url), shell=True)



"""Entry point for ecoshard."""
import os
import sys
import logging

import taskgraph
from flask import Flask
import flask

APP = Flask(__name__)
APP.config['SECRET_KEY'] = b'\xe2\xa9\xd2\x82\xd5r\xef\xdb\xffK\x97\xcfM\xa2WH'
VISITED_POINT_ID_TIMESTAMP_MAP = {}
ACTIVE_USERS_MAP = {}
WORKSPACE_DIR = 'workspace_ecoshard_visualizer'
DATA_STORE_DIR = 'datastore'
VALIDATION_DATABASE_PATH = os.path.join(WORKSPACE_DIR)
DATABASE_PATH = os.path.join(WORKSPACE_DIR, 'dam_bounding_box_db.db')
N_WORKERS = -1
REPORTING_INTERVAL = 5.0

LOGGER = logging.getLogger(__name__)

logging.basicConfig(
    level=logging.DEBUG,
    format=('%(message)s'),
    stream=sys.stdout)
logging.getLogger('ecoshard').setLevel(logging.WARN)
LOGGER = logging.getLogger(__name__)

@APP.route('/favicon.ico')
def favicon():
    return flask.send_from_directory(
        os.path.join(APP.root_path, 'images'), 'favicon.ico',
        mimetype='image/vnd.microsoft.icon')

@APP.route('/<string:ecoshard_id>/<string:style_id>')
def ecoshard_with_style(ecoshard_id, style_id):
    """Entry point."""
    LOGGER.info('ecoshard with style: %s %s', ecoshard_id, style_id)
    datastore_id = ecoshard_id.split(':')[0]
    layer_id = ecoshard_id
    return flask.render_template(
        'ecoshard.html', **{
            'datastore_id': datastore_id,
            'layer_id': layer_id,
            'style_id': style_id,
        })

@APP.route('/<string:ecoshard_id>')
def ecoshard(ecoshard_id):
    """Entry point."""
    LOGGER.info('ecoshard as: %s', ecoshard_id)
    datastore_id = ecoshard_id.split(':')[0]
    layer_id = ecoshard_id
    return flask.render_template(
        'ecoshard.html', **{
            'datastore_id': datastore_id,
            'layer_id': layer_id,
            'style_id': '',
        })


if __name__ == '__main__':
    DB_CONN_THREAD_MAP = {}
    TASK_GRAPH = taskgraph.TaskGraph(
        WORKSPACE_DIR, N_WORKERS, reporting_interval=REPORTING_INTERVAL)
    APP.run(host='0.0.0.0', port=8888)
    # this makes a connection per thread



"""ecoshard setup.py."""
from setuptools import setup

LONG_DESCRIPTION = '%s\n\n%s' % (
    open('README.rst').read(),
    open('HISTORY.rst').read())

REQUIREMENTS = [
    x for x in open('requirements.txt').read().split('\n')
    if not x.startswith('#') and len(x) > 0]

setup(
    name='ecoshard',
    use_scm_version={'version_scheme': 'post-release',
                     'local_scheme': 'node-and-date'},
    setup_requires=['setuptools_scm'],
    install_requires=REQUIREMENTS,
    description='EcoShard GIS data',
    long_description=LONG_DESCRIPTION,
    maintainer='Rich Sharp',
    maintainer_email='richpsharp@gmail.com',
    url='https://bitbucket.org/richsharp/ecoshard',
    packages=['ecoshard'],
    package_dir={
        'ecoshard': 'src/ecoshard',
    },
    zip_safe=False,
    include_package_data=True,
    license='BSD',
    keywords='computing reproduction',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: BSD License'
    ])



"""Build CV Styles."""
import logging
import sys

from jinja2 import Environment, FileSystemLoader
import numpy
from osgeo import ogr

LOGGER = logging.getLogger(__name__)

logging.basicConfig(
    level=logging.DEBUG,
    format=('%(message)s'),
    stream=sys.stdout)
logging.getLogger('ecoshard').setLevel(logging.WARN)
LOGGER = logging.getLogger(__name__)

FIELD_NAMES = [
    'coastal_deficit_cur',
    'coastal_deficit_ssp1',
    'coastal_deficit_ssp3',
    'coastal_deficit_ssp5',
    'coastal_NC_cur',
    'coastal_NC_ssp1',
    'coastal_NC_ssp3',
    'coastal_NC_ssp5',
    'coastal_potential_cur',
    'coastal_potential_ssp1',
    'coastal_potential_ssp3',
    'coastal_potential_ssp5',
    'coastal_pop_cur',
    'coastal_pop_ssp1',
    'coastal_pop_ssp3',
    'coastal_pop_ssp5',
    'coastal_deficit_change_ssp1',
    'coastal_deficit_change_ssp3',
    'coastal_deficit_change_ssp5',
    'coastal_NC_change_ssp1',
    'coastal_NC_change_ssp3',
    'coastal_NC_change_ssp5',
    'coastal_potential_change_ssp1',
    'coastal_potential_change_ssp3',
    'coastal_potential_change_ssp5',
    'coastal_pop_change_ssp1',
    'coastal_pop_change_ssp3',
    'coastal_pop_change_ssp5',
]

CV_VECTOR_PATH = '../cv_coastal_points_ipbes_md5_d1cc9481caf145c04fc14679265ed459.gpkg'


def main():
    """Entry point."""
    vector = ogr.Open(CV_VECTOR_PATH)
    layer = vector.GetLayer()
    layer_iter = iter(layer)
    first_feature = next(layer_iter)
    min_max_field_map = {}
    for field_name in FIELD_NAMES:
        min_max_field_map[field_name] = {
            'min': first_feature.GetField(field_name),
            'max': first_feature.GetField(field_name),
        }
    LOGGER.debug("process layer")
    for feature in layer_iter:
        for field_name in FIELD_NAMES:
            val = feature.GetField(field_name)
            if val < min_max_field_map[field_name]['min']:
                min_max_field_map[field_name]['min'] = val
            elif val > min_max_field_map[field_name]['max']:
                min_max_field_map[field_name]['max'] = val

    LOGGER.debug('write sld files')
    for field_name in FIELD_NAMES:
        env = Environment(loader=FileSystemLoader('.'))
        template = env.get_template('cv_style_template.xml')
        steps = numpy.linspace(
            min_max_field_map[field_name]['min'],
            min_max_field_map[field_name]['max'],
            6)
        output_from_parsed_template = template.render(
            field_name=field_name,
            step_1=steps[0],
            step_2=steps[1],
            step_3=steps[2],
            step_4=steps[3],
            step_5=steps[4],
            step_6=steps[5])
        # to save the results
        with open("%s.sld" % field_name, "w") as fh:
            fh.write(output_from_parsed_template)


if __name__ == '__main__':
    main()
